//+------------------------------------------------------------------+
//|                                                 SSS_EA_beta2.mq4 |
//|                                                         Zen_Leow |
//|                                                                  |
//+------------------------------------------------------------------+
/*
Chart setup: 
EMA 5 close
EMA 8 open
MomentumVT 10
QQE Smooth Factor 1

1)Buy: 5/8 ma up crossover
MomentumVT above 0 level
QQE crossing up

Exit buy: When the 5ema cross down across the 8ema and the QQE has crossed down. Until both occur, there is no exit. 

2)Sell: 5/8 ma down crossover
MomentumVT below 0
QQE crossing down

Exit sell: When the 5ema cross up across the 8ema and the QQE has crossed up. Until both occur, there is no exit. 

It only trade from 8:00 gmt to 18:00 gmt but it would be better if the time settings could be customizeable.

It wait for the candle to close for a confirmed signal.

The signals don't have to occur at the same time. buffer can be set as 15 bars.

MomentumVT should just be below/above the zero line and not neccesarily cross from above to below and vise versa.

For QQE buy, solid line cross up over dotted line i.e solid line above, dotted line below. For sell, solid line cross down below dotted line i.e solid line below, dotted line above.

--------------------------------------------------------------------------------------------------
What's new in beta 2?
=====================

1. separate entry and exit signal buffers for EMA and QQE.

2. Fixed logic error in Update_Status() which didn't work when there's only one trade in play.
   (breakeven and trailing stop should work now)
   
3. Fixed minor errors in Update_Status(), should use MODE_ASK instead of MODE_BID when closing a sell position.

4. Fixed logic errors which didn't allow trailing stop when BreakevenPip was set to 0.
   
5. Fixed bug which cause opening in opposite direction when there's already a trade running.

6. Added additional comments on top left corner of chart.
*/

#property copyright "Zen_Leow"
#property link      ""
#include <stdlib.mqh>
#include <stderror.mqh>

extern int       EA_MAGIC_NUM = 694373;
extern int       iOpenHour = 7;  // time the orders are placed 
extern int       HoursToTrade = 10;  // active hours to monitor 
extern string    Title1 = "Signal Configuration";
extern int       EMA_Close_Period = 5;
extern int       EMA_Open_Period = 8;
extern int       MomentumVT_Input = 10;
extern int       QQE_Smooth_Factor = 1;
extern int       EMA_Entry_Signal_Buffer = 5;
extern int       QQE_Entry_Signal_Buffer = 5; 
extern int       EMA_Exit_Signal_Buffer = 15;
extern int       QQE_Exit_Signal_Buffer = 15;
extern bool      Wait_For_Candle_Close = true;

extern string    Title2 = "Trade Management";
extern int       Slippage = 3;
extern int       UserDefinedSpread = 0;
extern int       TakeProfit = 200;
extern int       StopLoss = 30;
extern int       BreakEvenPips = 25;
extern int       TrailingStop = 20;
extern bool      UseFloatingStop = true;
extern bool      MoneyManagement = true;
extern double    RiskPercent = 1.5;
extern double    Lots = 1.0;
extern int       LotsDecimalAllowed = 2;
extern double    MaxLots = 15.0;
extern double    MinLots = 0.01;

string msg = "";
int iCloseHour;
int SignalIndex = 1;
int CustomSpread = 0;
//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
{
//----
   if (UserDefinedSpread <= 0)
   {
      CustomSpread = MarketInfo(Symbol(),MODE_SPREAD);
   }
   else
   {
      CustomSpread = UserDefinedSpread;
   }
   bool StartupStatusOK = true;
   if (MinLots < MarketInfo(Symbol(),MODE_MINLOT))
   {
      MinLots = MarketInfo(Symbol(),MODE_MINLOT);
      Alert("MinLots adjusted to: "+MinLots);
      StartupStatusOK = false;
   }
   if (MaxLots > MarketInfo(Symbol(),MODE_MAXLOT))
   {
      MaxLots = MarketInfo(Symbol(),MODE_MAXLOT);
      Alert("MaxLots adjusted to: "+MaxLots);
      StartupStatusOK = false;
   }
   if (iOpenHour < 0 || iOpenHour > 23)
   {
      iOpenHour = 0;
      Alert("Invalid OpeningHour given, adjusted to: "+iOpenHour);
      StartupStatusOK = false;
   }
   
   if (HoursToTrade < 1 || HoursToTrade > 24)
   {
      HoursToTrade = 10;
      Alert("Invalid HoursToTrade given, adjusted to: "+HoursToTrade);
      StartupStatusOK = false;
   }
   
   if (MoneyManagement && RiskPercent <= 0)
   {
      RiskPercent = 1.5;
      Alert("Invalid RiskPercent given, adjusted to: "+RiskPercent+"%");
      StartupStatusOK = false;
   }
   if (StartupStatusOK)
   {
      Alert("EA initiated with valid inputs");
   }
   iCloseHour = iOpenHour + HoursToTrade;
   if (iCloseHour >= 24)
   {
      iCloseHour = iCloseHour - 24;
   }
   if (Wait_For_Candle_Close)
   {
      SignalIndex = 1;
   }
   else
   {
      SignalIndex = 0;
   }
   WriteComment();
//----
   return(0);
}
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
{
//----
   
//----
   return(0);
}

void GetSpread()
{
   if (UserDefinedSpread <= 0)
   {
      CustomSpread = MarketInfo(Symbol(),MODE_SPREAD);
   }
   else
   {
      CustomSpread = UserDefinedSpread;
   }
}

void WriteComment()
{
   msg = "";
   int CurrentHour = TimeHour(TimeCurrent());
   // Only monitor a user defined period of the day
   if ((CurrentHour >= iOpenHour && CurrentHour < iCloseHour) ||  HoursToTrade == 24)
   {
      msg = msg + "EA working (looking for signals).";
   }
   else
   {
      msg = msg + "EA sleeping (but still managing opened trades if any).";
   }
   if (MoneyManagement && StopLoss != 0)
   {
      msg = msg + "\nUsing MoneyManagement Feature according to Stop loss";
   }
   if (MoneyManagement && StopLoss <= 0)
   {
      msg = msg + "\nUsing MoneyManagement Feature without using Stop loss";
   }
   if (UseFloatingStop && TrailingStop != 0)
   {
      msg = msg + "\nUsing Floating Stop.";
   }
   if (!UseFloatingStop && TrailingStop != 0)
   {
      msg = msg + "\nUsing Jumping Stop.";
   }
   if (TrailingStop == 0)
   {
      msg = msg + "\nNot using trailing stop.";
   }
   Comment(msg);
}

double Get_EMA_Close_Value(int index)
{
   return (iMA(NULL, 0, EMA_Close_Period, 0, MODE_EMA, PRICE_CLOSE, index));
}

double Get_EMA_Open_Value(int index)
{
   return (iMA(NULL, 0, EMA_Open_Period, 0, MODE_EMA, PRICE_OPEN, index));
}

double Get_MomentumVT_Value(int index)
{
   return (iCustom(NULL, 0, "MomentumVT", MomentumVT_Input, 0, index));
}

double Get_QQE_Solid_Value(int index)
{
   return (iCustom(NULL, 0, "QQE", QQE_Smooth_Factor, 0, index));
}

double Get_QQE_Dotted_Value(int index)
{
   return (iCustom(NULL, 0, "QQE", QQE_Smooth_Factor, 1, index));
}

bool EMA_SaysBuy()
{
   bool shouldBuy = false;
   if (Get_EMA_Close_Value(SignalIndex) > Get_EMA_Open_Value(SignalIndex))
   {
      for (int i=SignalIndex+1; i<SignalIndex+EMA_Entry_Signal_Buffer; i++)
      {
         if (Get_EMA_Close_Value(i) < Get_EMA_Open_Value(i))
         {
            shouldBuy = true;
         }
      }
      if (EMA_Entry_Signal_Buffer == 0)
      {
         shouldBuy = true;
      }
   }
   
   return (shouldBuy);
}

bool EMA_SaysSell()
{
   bool shouldSell = false;
   if (Get_EMA_Close_Value(SignalIndex) < Get_EMA_Open_Value(SignalIndex))
   {
      for (int i=SignalIndex+1; i<SignalIndex+EMA_Entry_Signal_Buffer; i++)
      {
         if (Get_EMA_Close_Value(i) > Get_EMA_Open_Value(i))
         {
            shouldSell = true;
         }
      }
      if (EMA_Entry_Signal_Buffer == 0)
      {
         shouldSell = true;
      }
   }
   return (shouldSell);
}

bool EMA_SaysBuy_Exit()
{
   bool shouldBuy = false;
   if (Get_EMA_Close_Value(SignalIndex) > Get_EMA_Open_Value(SignalIndex))
   {
      for (int i=SignalIndex+1; i<SignalIndex+EMA_Exit_Signal_Buffer; i++)
      {
         if (Get_EMA_Close_Value(i) < Get_EMA_Open_Value(i))
         {
            shouldBuy = true;
         }
      }
      if (EMA_Exit_Signal_Buffer == 0)
      {
         shouldBuy = true;
      }
   }
   
   return (shouldBuy);
}

bool EMA_SaysSell_Exit()
{
   bool shouldSell = false;
   if (Get_EMA_Close_Value(SignalIndex) < Get_EMA_Open_Value(SignalIndex))
   {
      for (int i=SignalIndex+1; i<SignalIndex+EMA_Exit_Signal_Buffer; i++)
      {
         if (Get_EMA_Close_Value(i) > Get_EMA_Open_Value(i))
         {
            shouldSell = true;
         }
      }
      if (EMA_Exit_Signal_Buffer == 0)
      {
         shouldSell = true;
      }
   }
   return (shouldSell);
}

bool MomentumVT_SaysBuy()
{
   if (Get_MomentumVT_Value(SignalIndex) > 0)
   {
      return (true);
   }
   return (false);
}

bool MomentumVT_SaysSell()
{
   if (Get_MomentumVT_Value(SignalIndex) < 0)
   {
      return (true);
   }
   return (false);
}

bool QQE_SaysBuy()
{
   bool shouldBuy = false;
   if (Get_QQE_Solid_Value(SignalIndex) > Get_QQE_Dotted_Value(SignalIndex))
   {
      for (int i=SignalIndex+1; i<SignalIndex+QQE_Entry_Signal_Buffer; i++)
      {
         if (Get_QQE_Solid_Value(i) < Get_QQE_Dotted_Value(i))
         {
            shouldBuy = true;
         }
      }
      if (QQE_Entry_Signal_Buffer == 0)
      {
         shouldBuy = true;
      }
   }
   return (shouldBuy);
}

bool QQE_SaysSell()
{
   bool shouldSell = false;
   if (Get_QQE_Solid_Value(SignalIndex) < Get_QQE_Dotted_Value(SignalIndex))
   {
      for (int i=SignalIndex+1; i<SignalIndex+QQE_Entry_Signal_Buffer; i++)
      {
         if (Get_QQE_Solid_Value(i) > Get_QQE_Dotted_Value(i))
         {
            shouldSell = true;
         }
      }
      if (QQE_Entry_Signal_Buffer == 0)
      {
         shouldSell = true;
      }
   }
   return (shouldSell);
}

bool QQE_SaysBuy_Exit()
{
   bool shouldBuy = false;
   if (Get_QQE_Solid_Value(SignalIndex) > Get_QQE_Dotted_Value(SignalIndex))
   {
      for (int i=SignalIndex+1; i<SignalIndex+QQE_Exit_Signal_Buffer; i++)
      {
         if (Get_QQE_Solid_Value(i) < Get_QQE_Dotted_Value(i))
         {
            shouldBuy = true;
         }
      }
      if (QQE_Exit_Signal_Buffer == 0)
      {
         shouldBuy = true;
      }
   }
   return (shouldBuy);
}

bool QQE_SaysSell_Exit()
{
   bool shouldSell = false;
   if (Get_QQE_Solid_Value(SignalIndex) < Get_QQE_Dotted_Value(SignalIndex))
   {
      for (int i=SignalIndex+1; i<SignalIndex+QQE_Exit_Signal_Buffer; i++)
      {
         if (Get_QQE_Solid_Value(i) > Get_QQE_Dotted_Value(i))
         {
            shouldSell = true;
         }
      }
      if (QQE_Exit_Signal_Buffer == 0)
      {
         shouldSell = true;
      }
   }
   return (shouldSell);
}

bool ShouldBuy()
{
   if (EMA_SaysBuy() && MomentumVT_SaysBuy() && QQE_SaysBuy())
   {
      return (true);
   }
   return (false);
}

bool ShouldSell()
{
   if (EMA_SaysSell() && MomentumVT_SaysSell() && QQE_SaysSell())
   {
      return (true);
   }
   return (false);
}

bool DecideToOpenTrade()
{
   int total = OrdersTotal();
   if (total > 0)
   {
      for(int cnt=0;cnt<total;cnt++)
      {
         if(OrderSelect(cnt,SELECT_BY_POS))
         {
            if(OrderSymbol()==Symbol() && OrderMagicNumber() == EA_MAGIC_NUM)
            {
               return (false);
            }
         }
      }
   }
   // in case trades has already opened and closed within the candle
   int histotal = OrdersHistoryTotal();
   if (histotal > 0)
   {
      for(cnt=0;cnt<histotal;cnt++)
      {
         if(OrderSelect(cnt,SELECT_BY_POS,MODE_HISTORY))
         {
            if(OrderSymbol()==Symbol() && OrderMagicNumber() == EA_MAGIC_NUM)
            {
               if (Time[0] <= OrderOpenTime()) // don't open a new position if we're still on the same candle
               {
                  return (false);
               }
            }
         }
      }
   }
   return (true);
}

double PositionSizeToOpen() // only works for pairs with JPY, CHF, CAD, GBP, NZD, AUD and USD as quote currency (apologies to exotic pairs traders)
{
   double PositionSize;
   int StopLossPips = StopLoss; // redundant assignment.. but I'm lazy to change all the variables below. =p
   if (MoneyManagement && StopLossPips > 0)
   {      
      double riskDollars = (AccountBalance()/100)*RiskPercent;
      double USDCHFBid, USDJPYBid, USDCADBid, GBPUSDBid, AUDUSDBid, NZDUSDBid;
      
      string mini = "";
      
      if (StringLen(Symbol())>6) // possibly a mini account
      {
         if (StringSubstr(Symbol(), 6, 1)=="m")
         {
            mini = "m";
         }
      }
            
      if (StringSubstr(Symbol(), 3, 3)=="USD")
      {
         PositionSize = riskDollars / ((StopLossPips + CustomSpread) * Point) / 100000;
      }
      
      if (StringSubstr(Symbol(), 3, 3)=="CHF")
      {
         USDCHFBid = iClose("USDCHF"+mini,PERIOD_D1,1); // get yesterday's exchange rate between USD and CHF
         PositionSize = riskDollars / (StopLossPips * Point * (1/USDCHFBid)) / 100000;
      }
      
      if (StringSubstr(Symbol(), 3, 3)=="JPY")
      {
         USDJPYBid = iClose("USDJPY"+mini,PERIOD_D1,1); // get yesterday's exchange rate between USD and JPY
         PositionSize = riskDollars / ((StopLossPips + CustomSpread) * Point * (1/USDJPYBid)) / 100000;
      }
      
      if (StringSubstr(Symbol(), 3, 3)=="CAD")
      {
         USDCADBid = iClose("USDCAD"+mini,PERIOD_D1,1); // get yesterday's exchange rate between USD and CAD
         PositionSize = riskDollars / ((StopLossPips + CustomSpread) * Point * (1/USDCADBid)) / 100000;
      }
      
      if (StringSubstr(Symbol(), 3, 3)=="GBP")
      {
         GBPUSDBid = iClose("GBPUSD"+mini,PERIOD_D1,1); // get yesterday's exchange rate between GBP and USD
         PositionSize = riskDollars / ((StopLossPips + CustomSpread) * Point * GBPUSDBid) / 100000;
      }
      
      if (StringSubstr(Symbol(), 3, 3)=="NZD")
      {
         NZDUSDBid = iClose("NZDUSD"+mini,PERIOD_D1,1); // get yesterday's exchange rate between NZD and USD
         PositionSize = riskDollars / ((StopLossPips + CustomSpread) * Point * NZDUSDBid) / 100000;
      }
      
      if (StringSubstr(Symbol(), 3, 3)=="AUD")
      {
         AUDUSDBid = iClose("AUDUSD"+mini,PERIOD_D1,1); // get yesterday's exchange rate between NZD and USD
         PositionSize = riskDollars / ((StopLossPips + CustomSpread) * Point * AUDUSDBid) / 100000;
      }
   }
   
   if (MoneyManagement && StopLossPips <= 0)
   {
      PositionSize = ((AccountEquity()/100) * RiskPercent) / (MarketInfo(Symbol(),MODE_LOTSIZE) / AccountLeverage());
   }
   
   if (!MoneyManagement)
   {
      PositionSize = Lots;
   }
   
   if (PositionSize < MinLots)
   {
      PositionSize = MinLots;
   }
   if (PositionSize > MaxLots)
   {
      PositionSize = MaxLots;
   }
   PositionSize = NormalizeDouble(PositionSize,LotsDecimalAllowed);
   return (PositionSize);
}

bool SendOrders (int BuyOrSell, double LotSize, double PriceToOpen, double Slippage, double SL_Price, double TP_Price, string comments, datetime ExpirationTime)
{
   int ticket, errorType;
   
   if (BuyOrSell == OP_BUY)
   {  
      Print("Bid: "+Bid+" Ask: "+Ask+" | Opening Buy Order: "+Symbol()+", "+BuyOrSell+", "+LotSize+", "+PriceToOpen+", "+Slippage+", "+SL_Price+", "+TP_Price+", "+comments+", "+EA_MAGIC_NUM+", "+ExpirationTime+", Green");
      ticket=OrderSend(Symbol(),BuyOrSell,LotSize,PriceToOpen,Slippage,SL_Price,TP_Price,comments,EA_MAGIC_NUM,ExpirationTime,Green);
      if(ticket>0)
      {
         if(OrderSelect(ticket,SELECT_BY_TICKET,MODE_TRADES)) 
         {
            Print("BUY order opened : ",OrderOpenPrice());
            msg = ticket + ": Buy position opened on "+Symbol()+" at "+ Day()+"/"+Month()+"/"+Year()+" - "+Hour()+":"+Minute()+":"+Seconds();
            WriteToLogFile(msg);
            return (true);
         }
      }
      else 
      {  
         errorType = GetLastError();
         Print("Error opening BUY order : ", ErrorDescription(errorType));
         msg = "CANNOT open BUY position on "+Symbol()+" at "+ Day()+"/"+Month()+"/"+Year()+" - "+Hour()+":"+Minute()+":"+Seconds();
         WriteToLogFile(msg);
         return (false);
      }
   }
   if (BuyOrSell == OP_SELL)
   {  
      Print("Bid: "+Bid+" Ask: "+Ask+" | Opening Sell Order: "+Symbol()+", "+BuyOrSell+", "+LotSize+", "+PriceToOpen+", "+Slippage+", "+SL_Price+", "+TP_Price+", "+comments+", "+EA_MAGIC_NUM+", "+ExpirationTime+", Red");
      ticket=OrderSend(Symbol(),BuyOrSell,LotSize,PriceToOpen,Slippage,SL_Price,TP_Price,comments,EA_MAGIC_NUM,ExpirationTime,Red);
      if(ticket>0)
      {
         if(OrderSelect(ticket,SELECT_BY_TICKET,MODE_TRADES)) 
         {
            Print("Sell order opened : ",OrderOpenPrice());
            msg = ticket + ": Sell position opened on "+Symbol()+" at "+ Day()+"/"+Month()+"/"+Year()+" - "+Hour()+":"+Minute()+":"+Seconds();
            WriteToLogFile(msg);
            return (true);
         }
      }
      else 
      {  
         errorType = GetLastError();
         Print("Error opening SELL order : ", ErrorDescription(errorType));
         msg = "CANNOT open SELL position on "+Symbol()+" at "+ Day()+"/"+Month()+"/"+Year()+" - "+Hour()+":"+Minute()+":"+Seconds();
         WriteToLogFile(msg);
         return (false);
      }
   }
}

void UpdateStatus()
{
   int total = OrdersTotal()-1;
   if (total >= 0)
   {
      for(int cnt=total;cnt>=0;cnt--)
      {
         if(OrderSelect(cnt,SELECT_BY_POS))
         {
            if(OrderSymbol()==Symbol() && OrderMagicNumber() == EA_MAGIC_NUM)
            {
               if (OrderType() == OP_BUY)
               {
                  // if the price has moved in your direction for set number of pips, move stop loss level to break even.
                  if ((OrderOpenPrice() > OrderStopLoss()) && BreakEvenPips > 0)
                  {
                     if(Bid-OrderOpenPrice() >= BreakEvenPips*Point)
                     {
                        Print(OrderTicket()+" should be moved to breakeven now.");
                        OrderModify(OrderTicket(),OrderOpenPrice(),OrderOpenPrice(),OrderTakeProfit(),0,Green);
                        // if signal for opposite direction occurs, close the position.
                        if (EMA_SaysSell_Exit() && QQE_SaysSell_Exit())
                        {
                           OrderClose(OrderTicket(),OrderLots(),MarketInfo(OrderSymbol(),MODE_BID),3);
                           continue;
                        }
                        continue;
                     }
                  }
                  // After stoploss has moved to breakeven, start to use trailing stop
                  if (UseFloatingStop && TrailingStop != 0 && (OrderOpenPrice() <= OrderStopLoss() || BreakEvenPips <= 0))
                  {
                     if(Bid-OrderOpenPrice() > TrailingStop*Point)
                     {
                        if(OrderStopLoss() < Bid-TrailingStop*Point)
                        {
                           OrderModify(OrderTicket(),OrderOpenPrice(),Bid-TrailingStop*Point,OrderTakeProfit(),0,Green);
                           // if signal for opposite direction occurs, close the position.
                           if (EMA_SaysSell_Exit() && QQE_SaysSell_Exit())
                           {
                              OrderClose(OrderTicket(),OrderLots(),MarketInfo(OrderSymbol(),MODE_BID),3);
                              continue;
                           }
                           continue;
                        }
                     }
                  }
                  // Check for Jumping Stop
                  if (!UseFloatingStop && TrailingStop != 0 && (OrderOpenPrice() <= OrderStopLoss() || BreakEvenPips <= 0))
                  {
                     // only activate when position is in profit by stoploss amt.
                     if((Bid-OrderStopLoss() > (TrailingStop*Point*2)) && (Bid-OrderOpenPrice()>(OrderOpenPrice()-OrderStopLoss())))
                     {
                        if ((OrderStopLoss()+TrailingStop*Point) < OrderTakeProfit()) // Stop loss level cannot be higher than take profit level
                        {
                           OrderModify(OrderTicket(),OrderOpenPrice(),OrderStopLoss()+TrailingStop*Point,OrderTakeProfit(),0,Green);
                           // if signal for opposite direction occurs, close the position.
                           if (EMA_SaysSell_Exit() && QQE_SaysSell_Exit())
                           {
                              OrderClose(OrderTicket(),OrderLots(),MarketInfo(OrderSymbol(),MODE_BID),3);
                              continue;
                           }
                           continue;
                        }
                     }
                  }
                  // if signal for opposite direction occurs, close the position.
                  if (EMA_SaysSell_Exit() && QQE_SaysSell_Exit())
                  {
                     OrderClose(OrderTicket(),OrderLots(),MarketInfo(OrderSymbol(),MODE_BID),3);
                     continue;
                  }
               }
               
               if (OrderType() == OP_SELL)
               {
                  // if the price has moved in your direction for set number of pips, move stop loss level to break even.
                  if ((OrderOpenPrice() < OrderStopLoss()) && BreakEvenPips > 0)
                  {
                     if(OrderOpenPrice()-Ask >= BreakEvenPips*Point)
                     {
                        Print(OrderTicket()+" should be moved to breakeven now.");
                        OrderModify(OrderTicket(),OrderOpenPrice(),OrderOpenPrice(),OrderTakeProfit(),0,Red);
                        // if signal for opposite direction occurs, close the position.
                        if (EMA_SaysBuy_Exit() && QQE_SaysBuy_Exit())
                        {
                           OrderClose(OrderTicket(),OrderLots(),MarketInfo(OrderSymbol(),MODE_ASK),3);
                           continue;
                        }
                        continue;
                     }
                  }
                  
                  // Check for Floating Stop
                  if (UseFloatingStop && TrailingStop != 0 && (OrderOpenPrice() >= OrderStopLoss() || BreakEvenPips <= 0))
                  {
                     if((OrderOpenPrice()-Ask)>(TrailingStop*Point))
                     {
                        if((OrderStopLoss()>(Ask+TrailingStop*Point)))
                        {
                           OrderModify(OrderTicket(),OrderOpenPrice(),Ask+TrailingStop*Point,OrderTakeProfit(),0,Red);
                           // if signal for opposite direction occurs, close the position.
                           if (EMA_SaysBuy_Exit() && QQE_SaysBuy_Exit())
                           {
                              OrderClose(OrderTicket(),OrderLots(),MarketInfo(OrderSymbol(),MODE_ASK),3);
                              continue;
                           }
                           continue;
                        }
                     }
                  }
                  // Check for Jumping Stop
                  if (!UseFloatingStop && TrailingStop != 0 && (OrderOpenPrice() >= OrderStopLoss() || BreakEvenPips <= 0))
                  {
                     // only activate when position is in profit by stoploss amt.
                     if(((OrderStopLoss()-Ask)>(TrailingStop*Point*2)) && ((OrderOpenPrice()-Ask)>(OrderStopLoss()-OrderOpenPrice())))
                     {
                        if ((OrderStopLoss()-TrailingStop*Point)>OrderTakeProfit()) // Stop loss level cannot be less than take profit level
                        {
                           OrderModify(OrderTicket(),OrderOpenPrice(),OrderStopLoss()-TrailingStop*Point,OrderTakeProfit(),0,Red);
                           // if signal for opposite direction occurs, close the position.
                           if (EMA_SaysBuy_Exit() && QQE_SaysBuy_Exit())
                           {
                              OrderClose(OrderTicket(),OrderLots(),MarketInfo(OrderSymbol(),MODE_ASK),3);
                              continue;
                           }
                           continue;
                        }
                     }
                  }
                  // if signal for opposite direction occurs, close the position.
                  if (EMA_SaysBuy_Exit() && QQE_SaysBuy_Exit())
                  {
                     OrderClose(OrderTicket(),OrderLots(),MarketInfo(OrderSymbol(),MODE_ASK),3);
                     continue;
                  }
               }
            }
         }
      }
   }
}

void WriteToLogFile(string input)
{
   string filename = "SSS_EA-"+Symbol()+"-"+Day()+"-"+Month()+"-"+Year()+".log";
   int handle = FileOpen(filename,FILE_READ|FILE_WRITE);
   if (handle>1)
   {
      FileSeek(handle, 0, SEEK_END); // go to end of file
      FileWrite(handle, input);
      FileClose(handle);
   }
}

//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
{
//----
   UpdateStatus();
   GetSpread();
   WriteComment();
   int CurrentHour = TimeHour(TimeCurrent());
   // Only monitor a user defined period of the day
   if ((CurrentHour >= iOpenHour && CurrentHour < iCloseHour) ||  HoursToTrade == 24)
   {
      double PriceToOpen;
      double TakeProfitPrice;
      double StopLossPrice;
      
      if (ShouldBuy())
      {
         if (DecideToOpenTrade())
         {
            PriceToOpen = Ask;
              
            if (TakeProfit == 0)
            {
               TakeProfitPrice = 0;
            }
            else
            {
               TakeProfitPrice = PriceToOpen + (TakeProfit * Point);
               TakeProfitPrice = NormalizeDouble(TakeProfitPrice,Digits);
            }
            if (StopLoss == 0)
            {
               StopLossPrice = 0;
            }
            else
            {
               StopLossPrice = PriceToOpen - ((StopLoss+CustomSpread) * Point);
               StopLossPrice = NormalizeDouble(StopLossPrice,Digits);
            }
            SendOrders(OP_BUY, PositionSizeToOpen(), PriceToOpen, Slippage, StopLossPrice, TakeProfitPrice, "SSS_Buy_"+EA_MAGIC_NUM, 0);
         }
      }
      
      if (ShouldSell())
      {
         if (DecideToOpenTrade())
         {
            PriceToOpen = Bid;
              
            if (TakeProfit == 0)
            {
               TakeProfitPrice = 0;
            }
            else
            {
               TakeProfitPrice = PriceToOpen - (TakeProfit * Point);
               TakeProfitPrice = NormalizeDouble(TakeProfitPrice,Digits);
            }
            if (StopLoss == 0)
            {
               StopLossPrice = 0;
            }
            else
            {
               StopLossPrice = PriceToOpen + ((StopLoss+CustomSpread) * Point);
               StopLossPrice = NormalizeDouble(StopLossPrice,Digits);
            }
            SendOrders(OP_SELL, PositionSizeToOpen(), PriceToOpen, Slippage, StopLossPrice, TakeProfitPrice, "SSS_Buy_"+EA_MAGIC_NUM, 0);
         }
      }
   }
   
//----
   return(0);
}
//+------------------------------------------------------------------+